#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
CLINGON
Command Line INterpreter Generator for pythON
Compatible python 2 and 3
(c) François Vincent, https://github.com/francois-vincent
"""

from __future__ import print_function, absolute_import
from future.utils import listitems, iteritems
from past.builtins import basestring

from collections import Sequence
try:
    from collections import OrderedDict
except ImportError:
    from ordereddict import OrderedDict
import inspect
import os
import sys
import textwrap

__version__ = '0.1.1'
CLINGON_VERSION = __version__
DEBUG = False
TEST = False
SYSTEM_EXIT_ERROR_CODE = 1


class Clizer(object):
    """
    This virtual class extracts args off the run() method of any derived subclass
    Then it extracts cmd line arguments and launches run() with matched args
    """
    _help_options = ('--help', '-?')
    _version_options = ('--version', '-V')

    class RunnerError(RuntimeError):
        pass

    class RunnerErrorWithUsage(RuntimeError):
        pass

    @staticmethod
    def _write_error(*args, **kwargs):
        sys.stderr.write(kwargs.get('sep', ' ').join(args) + kwargs.get('help', ' (-? for help)')
                         + kwargs.get('end', '\n'))

    @staticmethod
    def _get_type(arg):
        return type(arg).__name__

    @classmethod
    def _sys_exit(cls, code=SYSTEM_EXIT_ERROR_CODE):
        if cls.SYSTEM_EXIT:
            if type(code) is not int:
                code = 0
            if DEBUG:
                print("Exit with code %d" % code)
            sys.exit(code)
        return code

    @classmethod
    def check_deco_parameters(cls, *args, **kwargs):
        if (args and kwargs) or len(args) > 1:
            raise ValueError("This decorator is for a function only")
        if args and not inspect.isfunction(args[0]):
            raise ValueError("This decorator is for a function only")
        if kwargs:
            for k, v in kwargs.items():
                if not isinstance(v, Sequence):
                    raise ValueError(
                        "Decorator's keyword '%s' value must be a string or a tuple of strings, found: %s" % (k, v))

    def __init__(self, func):
        self.func = func
        self.docstring = func.__doc__
        self.file = inspect.getfile(func)
        self.variables = getattr(func, 'variables', {})
        argspec = inspect.getargspec(self.func)
        # do not allow keywords
        if argspec.keywords:
            raise TypeError("Keywords parameter '**%s' is not allowed" % argspec.keywords)
        defaults = argspec.defaults or ()
        # get varargs
        self.varargs = argspec.varargs
        # get required args as a list and optional args as a dict (with default values)
        nb_args, len_defaults = len(argspec.args), len(defaults)
        self.reqargs = argspec.args[:nb_args - len_defaults]
        options = OrderedDict(zip((x.lower() for x in argspec.args[nb_args - len_defaults:]), defaults))
        # make a copy of options for later call of user's decorated function
        self.python_options = OrderedDict(options)
        # make an equivalence dict from line cmd style (--file-name) to python style (file_name) args
        self.options_equ = dict([('-' + x if len(x) == 1 else '--' + '-'.join(x.split('_')), x) for x in options])
        # make a dict of cmd line style arg names to their types
        self.options = OrderedDict(
            [('-' + x if len(x) == 1 else '--' + '-'.join(x.split('_')), options[x]) for x in options])
        # take a copy of original (no aliases yet) optional args for print_help()
        self._options = OrderedDict(self.options)
        # create automatic short options aliases from long options
        self.options_aliases = getattr(self, 'options_aliases', {})
        mismatch = set(self.options_aliases) - set(options)
        if mismatch:
            raise ValueError("This option does not exists so can't be given an alias: " + mismatch.pop())
        for k in options:
            if k not in self.options_aliases and len(k) > 1:
                self.options_aliases[k] = (k[0],)
            # inject aliases into dicts
        for x, t in options.items():
            if x in self.options_aliases:
                alias = self.options_aliases[x]
                if isinstance(alias, basestring):
                    alias = self.options_aliases[x] = (alias,)
                for a in alias:
                    k = '-' + a
                    if k in self.options or k in self._version_options:
                        # silently ignore duplicate short alias
                        continue
                    self.options[k] = t
                    self.options_equ[k] = x

    def _eval_global(self):
        for k, v in listitems(self.variables):
            self.variables[k] = v() if hasattr(v, '__call__') else v

    def _get_global(self, key):
        return self.variables.get(key)

    def _print_version(self):
        source = os.path.dirname(self.file)
        source = ' from ' + source if source else ''
        Clizer._write_error('version %s%s (Python %s)' % (self._get_global('VERSION'), source,
                                                   sys.version.split()[0]))

    def start(self, param_string=None):
        optargs = {}
        reqargs, varargs = [], []
        # first, search cmd line options and their args
        # and collect them in optargs.
        # along this process, required and optional args are also collected
        # in reqargs and varargs
        if param_string is None:
            argv = sys.argv[1:]
        else:
            import shlex
            argv = shlex.split(param_string)
        self._eval_global()
        i = 0
        while i < len(argv):
            x = argv[i]
            if x in self._help_options:
                self._print_help()
                return
            if x in self._version_options and self._get_global('VERSION'):
                self._print_version()
                return
            if x in self.options:
                if self.options_equ[x] in optargs:
                    raise Clizer.RunnerError("Option '%s' found twice" % x)
                if isinstance(self.options[x], (list, tuple)):
                    argpos = i
                    optargs[self.options_equ[x]] = []
                    i += 1
                    while i < len(argv) and argv[i] not in self.options:
                        try:
                            optargs[self.options_equ[x]].append(type(self.options[x][0])(argv[i]))
                        except ValueError:
                            raise Clizer.RunnerError("Argument %d of option %s has wrong type (%s expected)" %
                                                     (i - argpos, x, self._format_type(self.options[x][0])))
                        i += 1
                    if not len(optargs[self.options_equ[x]]):
                        raise Clizer.RunnerError("Option '%s' should be followed by a list of %s" %
                                                 (x, self._format_type(self.options[x][0])))
                elif type(self.options[x]) is bool:
                    optargs[self.options_equ[x]] = True
                    i += 1
                else:
                    i += 1
                    if i >= len(argv) or argv[i] in self.options:
                        raise Clizer.RunnerError("Option '%s' should be followed by a %s" %
                                                 (x, self._format_type(self.options[x])))
                    try:
                        optargs[self.options_equ[x]] = type(self.options[x])(argv[i])
                    except ValueError:
                        raise Clizer.RunnerError("Argument of option %s has wrong type (%s expected)" %
                                                 (x, self._format_type(self.options[x])))
                    i += 1
            else:
                if len(reqargs) < len(self.reqargs):
                    reqargs.append(argv[i])
                elif self.varargs:
                    varargs.append(argv[i])
                else:
                    raise Clizer.RunnerErrorWithUsage("Unrecognized parameter or option '%s'" % argv[i])
                i += 1
        if self.reqargs and len(reqargs) < len(self.reqargs):
            raise Clizer.RunnerErrorWithUsage("Too few parameters (%d required)" % len(self.reqargs))
            # merge required, optional args and options in allargs
        options = OrderedDict(self.python_options)
        options.update(optargs)
        allargs = reqargs
        allargs.extend(options.values())
        allargs.extend(varargs)
        if DEBUG:
            print('clize call parameters:', allargs)
        # all parameters are filled
        return self.func(*allargs)

    def __call__(self, param_string=None):
        try:
            code = self.start(param_string)
        except Clizer.RunnerErrorWithUsage as e:
            self._print_usage(file=sys.stderr)
            Clizer._write_error(str(e), help='')
        except Clizer.RunnerError as e:
            Clizer._write_error(str(e))
        except Exception as e:
            Clizer._write_error(str(e), help='')
            if DEBUG:
                raise
        else:
            return self.__class__._sys_exit(code)
        return self.__class__._sys_exit()

    def _format_type(self, arg, post=''):
        if isinstance(arg, list):
            return '<list of %s>' % self._get_type(arg[0]) + post
        elif isinstance(arg, bool):
            return ''
        else:
            return '<%s>' % self._get_type(arg) + post

    def _format_aliases(self, option):
        option = self.options_equ[option]
        if option in self.options_aliases:
            return '| -' + ' | -'.join(self.options_aliases[option]) + ' '
        return ''

    def _format_option(self, option, opt_type, width=1):
        option_format = "{option:{width}} {shorts}{opt_type}{default}"
        option_dict = dict(
            option=option,
            width=width,
            shorts=self._format_aliases(option),
            opt_type=self._format_type(opt_type, ' '),
            default='(default=%r)' % opt_type
        )
        return option_format.format(**option_dict)

    def _print_usage(self, prepend='usage: ', file=sys.stdout):
        usage_format = "{prepend}{basename} {reqargs}{varargs}{options}{version}{help}"
        usage_dict = dict(
            prepend=prepend,
            basename=(os.path.basename(self.file)),
            reqargs=' '.join(self.reqargs) + ' ',
            varargs='[varargs] ' if self.varargs else '',
            version='[' + ' | '.join(self._version_options) + '] ' if self._get_global('VERSION') else '',
            help='[' + ' | '.join(self._help_options) + ']'
        )
        many_options = False
        if len(self._options) == 1 and len(list(self._options)[0]) < 9:
            # if only one option with a 'short' name: put everything in one line
            usage_dict.update(
                options='[' + self._format_option(*listitems(self._options)[0]) + '] ',
            )
        elif len(self._options) > 0:
            # else defer options description at the end of help output
            usage_dict.update(
                options='[options] ',
                version=''
            )
            many_options = True
        else:
            usage_dict.update(
                options='',
            )
        print(usage_format.format(**usage_dict), file=file)
        return many_options

    def _print_help(self):
        """
        Help is automatically generated from the __doc__ of the subclass if present
        and from the names of the args of run(). Therefore args names selection
        is more important than ever here !
        """
        options = self._print_usage('\n  ', file=sys.stdout)
        if self.docstring:
            print()
            doc_string = self.docstring.format(**self.variables)
            for doc in doc_string.split('\n'):
                doc = doc.strip()
                if len(doc) > 2:
                    print(textwrap.fill(textwrap.dedent(doc).strip(),
                                        width=80, initial_indent='  ',
                                        subsequent_indent='   '))
        print()
        if options:
            print('Options:')
            width = max(len(k) for k in self._options)
            for x, t in self._options.items():
                print(self._format_option(x, t, width))
            if self._get_global('VERSION'):
                print('{0:{1}}'.format(self._version_options[0], width), end=' ')
                print('| ' + ' | '.join(self._version_options[1:]), 'print version',
                      '(%s)' % self._get_global('VERSION'))
            print('{0:{1}}'.format(self._help_options[0], width), end=' ')
            print('| ' + ' | '.join(self._help_options[1:]), 'print this help')
            print()


def clize(*args, **kwargs):
    """
    Decorator has 2 usages:
    @clize
    def decorated(params):
       ...
    optional parameters aliases are autogenerated, conflicts are silently ignored

    @clize(aliases)
    def decorated(params)
       ...
    where aliases is a dictionary, key=python parameter of decorated function, value=tuple of aliases
    """

    kwargs = dict((k.lower(), v) for k, v in iteritems(kwargs))

    class mycli(Clizer):
        SYSTEM_EXIT = True
        options_aliases = kwargs

    mycli.check_deco_parameters(*args, **kwargs)

    def runner(function):
        return mycli(function) if TEST else mycli(function)()

    if args:
        return runner(args[0])
    else:
        return runner


def set_variables(**kwargs):
    def f(x):
        x.variables = kwargs
        return x
    return f


def make_script(target_path='', target_name='', user=False, make_link=False,
                force=False, remove=False, no_check_shebang=False, auto_install_clingon=False, *args):
    """v{VERSION}
    This script makes a command line script out of a python file.
    For example, 'clingon script.py' will copy or symlink script.py to:
    - <python-path>/script (default),,
    - <path>/script if --path is specfied,
    - ~/bin/script is --user is speified.
    and then set the script as executable (without the .py extension)
    """
    if args and auto_install_clingon:
        Clizer._write_error("You cannot specify a target script and --auto-install-clingon at the same time")
        return 1
    if not (args or auto_install_clingon):
        Clizer._write_error("You must specify either a target script or --auto-install-clingon")
        return 1
    if len(args) > 1:
        Clizer._write_error("You must specify a single target, found: %s" % str(args))
        return 1
    if user and target_path:
        Clizer._write_error("You cannot specify --path and --user at the same time")
        return 1
    source = os.path.abspath(args[0]) if args else __file__
    dest_dir = os.path.normpath(os.path.expanduser('~/bin' if user else target_path or os.path.dirname(sys.executable)))
    target = os.path.join(dest_dir,
                  target_name if target_name else os.path.splitext(os.path.basename(source))[0])
    target_exists = os.path.exists(target)
    if remove:
        if target_exists:
            os.unlink(target)
            print("Script '%s' removed" % target)
        else:
            print("Script '%s' not found, nothing to do" % target)
        return
    if not os.path.exists(source):
        Clizer._write_error("Could not find source '%s', aborting" % source, help='')
        return 1
    if DEBUG:
        print('Source, target:', source, target)

    if target_exists:
        def same_file_same_type(source, target):
            if os.path.islink(target):
                return make_link and os.path.samefile(source, target)
            else:
                return not make_link and (open(source, 'rb').read() == open(target, 'rb').read())
        if same_file_same_type(source, target):
            print("Target '%s' already created, nothing to do" % target)
            return
        elif not force:
            Clizer._write_error("Target '%s' already exists, aborting" % target, help='')
            return 1

    if not os.path.isdir(dest_dir):
        # Create directory but only if father dir exists and user has rights
        if os.access(os.path.basename(dest_dir), os.W_OK):
            os.system("mkdir %s" % dest_dir)
        else:
            Clizer._write_error("Target folder '%s' does not exist, and cannot create it, aborting" % dest_dir, help='')
            return 1

    if not no_check_shebang:
        # Check that file starts with python shebang (#!/usr/bin/env python)
        import re
        if not re.match(r'#!\s*/usr/bin/env\s+python', open(source).readline()):
            Clizer._write_error("Your script's first line should be '#!/usr/bin/env python', aborting", help='')
            return 1

    # Now it's time to copy or symlink file
    if target_exists:
        os.unlink(target)
    import stat
    perms = stat.S_IXUSR | stat.S_IXGRP
    if os.getuid() == 0:
        perms |= stat.S_IXOTH
    if make_link:
        st = os.stat(source)
        if st.st_mode & perms != perms:
            os.chmod(source, st.st_mode | perms)
        os.symlink(source, target)
        st = os.stat(target)
        os.chmod(target, st.st_mode | perms)
        print('Script %s has been symlinked to %s' % (source, target))
    else:
        import shutil
        shutil.copyfile(source, target)
        st = os.stat(target)
        os.chmod(target, st.st_mode | perms)
        print('Script %s has been copied to %s' % (source, target))
    # check PATH and advise user to update it if relevant
    path_env = os.environ.get('PATH')
    if not path_env or dest_dir not in path_env:
        print("Please add your local bin path [%s] to your environment PATH" % dest_dir)


if __name__ == '__main__':

    clize(make_link=('m', 's', 'l'), force=('f', 'o'), target_path='p', target_name='n')(
        set_variables(VERSION=CLINGON_VERSION)(
            make_script))



# -*- coding: utf-8 -*-

from __future__ import print_function
from contextlib import contextmanager
import mock

try:
    # for py26
    import unittest2 as unittest
except ImportError:
    import unittest

import sys

sys.path.insert(0, '..')

from clingon import clingon
clingon.TEST = True
clingon.DEBUG = False

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

try:
    from collections import OrderedDict
except ImportError:
    # for py26
    from ordereddict import OrderedDict


@contextmanager
def captured_output():
    try:
        sys.stdout, sys.stderr = StringIO(), StringIO()
        yield sys.stdout, sys.stderr
    finally:
        sys.stdout, sys.stderr = sys.__stdout__, sys.__stderr__

# ---------- here are decorated functions under test -------------
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

@clingon.clize
def clized_default_shorts(p1, p2,
                          first_option='default_value',
                          second_option=5,
                          third_option=[4, 3],
                          last_option=False):
    """Help docstring
    """
    print('%s %s %s %s %s %s' % (p1, p2, first_option, second_option, third_option, last_option))


@clingon.clize(first_option='1', second_option=('2', 's', 'so'))
def clized_spec_shorts(p1, p2,
                       first_option='default_value',
                       second_option=5,
                       third_option=[4, 3],
                       last_option=False):
    if last_option:
        return 12
    print('%s %s %s %s %s %s' % (p1, p2, first_option, second_option, third_option, last_option))


@clingon.clize
def clized_that_raises():
    raise RuntimeError('I just raise')


@clingon.clize
def clized_varargs(p1, p2,
                   option='default_value',
                   *varargs):
    print('%s %s %s %s' % (p1, p2, option, varargs))


def version():
    return '1.2.3'


@clingon.clize
@clingon.set_variables(VERSION=version, message="you can dynamically customize help message !")
def clized_variables(p1, p2, long_name_option='default_value'):
    """Help docstring v{VERSION}
       {message}
    """
    pass

@clingon.clize
@clingon.set_variables(VERSION='1.2.3', message="you can dynamically customize help message !")
def clized_variables_one_short(p1, p2, option='default_value'):
    """Help docstring v{VERSION}
       {message}
    """
    pass


# ---------- end of decorated functions under test --------------
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

class TestDecoratorBasic(unittest.TestCase):

    def test_decorator_multiple_args(self):
        with self.assertRaises(ValueError) as cm:
            clingon.clize(1, 2)
        message = cm.exception.args[0]
        self.assertEqual(message, "This decorator is for a function only")

    def test_decorator_args_not_function(self):
        with self.assertRaises(ValueError) as cm:
            clingon.clize(1)
        message = cm.exception.args[0]
        self.assertEqual(message, "This decorator is for a function only")

    def test_decorator_kwargs_not_sequence(self):
        with self.assertRaises(ValueError) as cm:
            clingon.clize(toto=1)
        message = cm.exception.args[0]
        self.assertEqual(message, "Decorator's keyword 'toto' value must be "
                                               "a string or a tuple of strings, found: 1")

    def test_kwargs(self):

        def clized(*varargs, **keywargs):
            pass

        with self.assertRaises(TypeError) as cm:
            clingon.clize(clized)
        message = cm.exception.args[0]
        self.assertEqual(message, "Keywords parameter '**keywargs' is not allowed")

    def test_bad_spec_option(self):

        def clized(option='default_value'):
            pass

        with self.assertRaises(ValueError) as cm:
            clingon.clize(bad_option='')(clized)
        message = cm.exception.args[0]
        self.assertEqual(message, "This option does not exists so can't be given an alias: bad_option")

    def test_internal_data(self):

        def clized(p1, p2,
                   first_option='default_value',
                   second_option=5,
                   third_option=[4, 3],
                   last_option=False,
                   *varargs):
            """docstring"""
            pass

        ret = clingon.clize(clized)
        self.assertEqual(ret.func.__name__, 'clized')
        self.assertEqual(ret.docstring, "docstring")
        self.assertEqual(ret.reqargs, ['p1', 'p2'])
        self.assertEqual(ret.varargs, 'varargs')
        self.assertDictEqual(ret.options, {
            '--first-option': 'default_value',
            '-f': 'default_value',
            '--second-option': 5,
            '-s': 5,
            '--third-option': [4, 3],
            '-t': [4, 3],
            '--last-option': False,
            '-l': False,
        })
        self.assertDictEqual(ret._options, {
            '--first-option': 'default_value',
            '--second-option': 5,
            '--third-option': [4, 3],
            '--last-option': False,
        })
        self.assertDictEqual(ret.options_aliases, {
            'first_option': ('f',),
            'second_option': ('s',),
            'third_option': ('t',),
            'last_option': ('l',)
        })
        self.assertDictEqual(ret.options_equ, {
            '--first-option': 'first_option',
            '-f': 'first_option',
            '--second-option': 'second_option',
            '-s': 'second_option',
            '--third-option': 'third_option',
            '-t': 'third_option',
            '--last-option': 'last_option',
            '-l': 'last_option'
        })
        self.assertEqual(ret.python_options,
                         OrderedDict([
                             ('first_option', 'default_value'),
                             ('second_option', 5),
                             ('third_option', [4, 3]),
                             ('last_option', False)
                         ]))

    def test_options_same_first_letter(self):

        def clized(option_1=False,
                   option_2=5):
            pass

        ret = clingon.clize(clized)
        self.assertDictEqual(ret.options, {
            '--option-1': False,
            '-o': False,
            '--option-2': 5,
        })
        self.assertDictEqual(ret.options_aliases, {
            'option_1': ('o',),
            'option_2': ('o',),
        })
        self.assertDictEqual(ret.options_equ, {
            '--option-1': 'option_1',
            '-o': 'option_1',
            '--option-2': 'option_2',
        })


@mock.patch('sys.exit')
class TestDecorator(unittest.TestCase):
    def test_default_no_option(self, sys_exit):
        with captured_output() as (out, err):
            clized_default_shorts('p1 p2')
        self.assertEqual(out.getvalue(),
                         "p1 p2 default_value 5 [4, 3] False\n")
        self.assertEqual(err.getvalue(), '')
        sys_exit.assert_called_with(0)

    def test_default_first_option(self, sys_exit):
        with captured_output() as (out, err):
            clized_default_shorts('p1 p2 --first-option specific_value')
        self.assertEqual(out.getvalue(),
                         "p1 p2 specific_value 5 [4, 3] False\n")
        self.assertEqual(err.getvalue(), '')
        sys_exit.assert_called_with(0)

    def test_default_second_option(self, sys_exit):
        with captured_output() as (out, err):
            clized_default_shorts('p1 p2 --second-option 10')
        self.assertEqual(out.getvalue(),
                         "p1 p2 default_value 10 [4, 3] False\n")
        self.assertEqual(err.getvalue(), '')
        sys_exit.assert_called_with(0)

    def test_default_third_option(self, sys_exit):
        with captured_output() as (out, err):
            clized_default_shorts('p1 p2 --third-option 16 9')
        self.assertEqual(out.getvalue(),
                         "p1 p2 default_value 5 [16, 9] False\n")
        self.assertEqual(err.getvalue(), '')
        sys_exit.assert_called_with(0)

    def test_default_last_option(self, sys_exit):
        with captured_output() as (out, err):
            clized_default_shorts('p1 p2 --last-option')
        self.assertEqual(out.getvalue(),
                         "p1 p2 default_value 5 [4, 3] True\n")
        self.assertEqual(err.getvalue(), '')
        sys_exit.assert_called_with(0)

    def test_default_all_options(self, sys_exit):
        with captured_output() as (out, err):
            clized_default_shorts(
                'p1 p2 --third-option 16 9 --second-option 10 --first-option specific_value --last-option')
        self.assertEqual(out.getvalue(),
                         "p1 p2 specific_value 10 [16, 9] True\n")
        self.assertEqual(err.getvalue(), '')
        sys_exit.assert_called_with(0)

    def test_default_all_options_short(self, sys_exit):
        with captured_output() as (out, err):
            clized_default_shorts('p1 p2 -t 16 9 -s 10 -f specific_value -l')
        self.assertEqual(out.getvalue(),
                         "p1 p2 specific_value 10 [16, 9] True\n")
        self.assertEqual(err.getvalue(), '')
        sys_exit.assert_called_with(0)

    def test_option_bad_type(self, sys_exit):
        with captured_output() as (out, err):
            clized_default_shorts('p1 p2 --second-option x')
        self.assertEqual(err.getvalue(),
                         "Argument of option --second-option has wrong type (<int> expected) (-? for help)\n")
        self.assertEqual(out.getvalue(), '')
        sys_exit.assert_called_with(clingon.SYSTEM_EXIT_ERROR_CODE)

    def test_option_short_bad_type(self, sys_exit):
        with captured_output() as (out, err):
            clized_default_shorts('p1 p2 -s x')
        self.assertEqual(err.getvalue(),
                         "Argument of option -s has wrong type (<int> expected) (-? for help)\n")
        self.assertEqual(out.getvalue(), '')
        sys_exit.assert_called_with(clingon.SYSTEM_EXIT_ERROR_CODE)

    def test_option_list_bad_type(self, sys_exit):
        with captured_output() as (out, err):
            clized_default_shorts('p1 p2 --third-option 1 y')
        self.assertEqual(err.getvalue(),
                         'Argument 2 of option --third-option has wrong type (<int> expected) (-? for help)\n')
        self.assertEqual(out.getvalue(), '')
        sys_exit.assert_called_with(clingon.SYSTEM_EXIT_ERROR_CODE)

    def test_option_list_no_value(self, sys_exit):
        with captured_output() as (out, err):
            clized_default_shorts('p1 p2 -t')
        self.assertEqual(err.getvalue(),
                         "Option '-t' should be followed by a list of <int> (-? for help)\n")
        self.assertEqual(out.getvalue(), '')
        sys_exit.assert_called_with(clingon.SYSTEM_EXIT_ERROR_CODE)

    def test_spec_option(self, sys_exit):
        with captured_output() as (out, err):
            clized_spec_shorts('p1 p2 -1 specific_value')
        self.assertEqual(out.getvalue(),
                         "p1 p2 specific_value 5 [4, 3] False\n")
        self.assertEqual(err.getvalue(), '')
        sys_exit.assert_called_with(0)

    def test_spec_option_again(self, sys_exit):
        with captured_output() as (out, err):
            clized_spec_shorts('p1 p2 -1 specific_value -so 12')
        self.assertEqual(out.getvalue(),
                         "p1 p2 specific_value 12 [4, 3] False\n")
        self.assertEqual(err.getvalue(), '')
        sys_exit.assert_called_with(0)

    def test_unknown_parameter(self, sys_exit):
        with captured_output() as (out, err):
            clized_default_shorts('p1 p2 p3')
        self.assertEqual(err.getvalue(),
                         "usage: test_clingon.py p1 p2 [options] [--help | -?]\n"
                         "Unrecognized parameter or option 'p3'\n")
        self.assertEqual(out.getvalue(), '')
        sys_exit.assert_called_with(clingon.SYSTEM_EXIT_ERROR_CODE)

    def test_too_few_parameters(self, sys_exit):
        with captured_output() as (out, err):
            clized_default_shorts('p1')
        self.assertEqual(err.getvalue(),
                         "usage: test_clingon.py p1 p2 [options] [--help | -?]\n"
                         "Too few parameters (2 required)\n")
        self.assertEqual(out.getvalue(), '')
        sys_exit.assert_called_with(clingon.SYSTEM_EXIT_ERROR_CODE)

    def test_duplicate_option(self, sys_exit):
        with captured_output() as (out, err):
            clized_default_shorts('p1 p2 -l --last-option')
        self.assertEqual(err.getvalue(),
                         "Option '--last-option' found twice (-? for help)\n")
        self.assertEqual(out.getvalue(), '')
        sys_exit.assert_called_with(clingon.SYSTEM_EXIT_ERROR_CODE)

    def test_option_missing_value(self, sys_exit):
        with captured_output() as (out, err):
            clized_default_shorts('p1 p2 -s')
        self.assertEqual(err.getvalue(),
                         "Option '-s' should be followed by a <int> (-? for help)\n")
        self.assertEqual(out.getvalue(), '')
        sys_exit.assert_called_with(clingon.SYSTEM_EXIT_ERROR_CODE)

    def test_return_integer(self, sys_exit):
        with captured_output() as (out, err):
            clized_spec_shorts('p1 p2 -l')
        self.assertEqual(out.getvalue(), '')
        self.assertEqual(err.getvalue(), '')
        sys_exit.assert_called_with(12)

    def test_debug(self, sys_exit):
        clingon.DEBUG = True
        try:
            with captured_output() as (out, err):
                clized_spec_shorts('p1 p2 -l')
        finally:
            clingon.DEBUG = False
        self.assertEqual(out.getvalue(), "clize call parameters: ['p1', 'p2', 'default_value', 5, [4, 3], True]\n"
                                         "Exit with code 12\n")
        self.assertEqual(err.getvalue(), "")
        sys_exit.assert_called_with(12)

    def test_help_output(self, sys_exit):
        with captured_output() as (out, err):
            clized_default_shorts('-?')
        output = out.getvalue()
        self.assertIn(clized_default_shorts.docstring.strip(), output)
        self.assertIn("Options:\n"
                      "--first-option  | -f <str> (default='default_value')\n"
                      "--second-option | -s <int> (default=5)\n"
                      "--third-option  | -t <list of int> (default=[4, 3])\n"
                      "--last-option   | -l (default=False)\n"
                      "--help          | -? print this help", output)
        self.assertEqual(err.getvalue(), '')
        sys_exit.assert_called_with(0)

    def test_raise_nodebug(self, sys_exit):
        with captured_output() as (out, err):
            clized_that_raises('')
        self.assertEqual(out.getvalue(), '')
        self.assertEqual(err.getvalue(), "I just raise\n")
        sys_exit.assert_called_with(clingon.SYSTEM_EXIT_ERROR_CODE)

    def test_raise_debug(self, sys_exit):
        clingon.DEBUG = True
        try:
            with captured_output() as (out, err):
                self.assertRaises(RuntimeError, clized_that_raises, '')
        finally:
            clingon.DEBUG = False
        self.assertEqual(out.getvalue(), "clize call parameters: []\n")
        self.assertEqual(err.getvalue(), "I just raise\n")
        sys_exit.assert_not_called()

    def test_varargs_novararg(self, sys_exit):
        with captured_output() as (out, err):
            clized_varargs('p1 p2 -o 123')
        self.assertEqual(out.getvalue(), "p1 p2 123 ()\n")
        self.assertEqual(err.getvalue(), '')
        sys_exit.assert_called_with(0)

    def test_varargs_varargs(self, sys_exit):
        with captured_output() as (out, err):
            clized_varargs('p1 p2 p3 p4 -o 123 --catched')
        self.assertEqual(out.getvalue(), "p1 p2 123 ('p3', 'p4', '--catched')\n")
        self.assertEqual(err.getvalue(), '')
        sys_exit.assert_called_with(0)

    def test_varargs_too_few_parameters(self, sys_exit):
        with captured_output() as (out, err):
            clized_varargs('p1')
        self.assertEqual(err.getvalue(),
                         "usage: test_clingon.py p1 p2 [varargs] [--option | -o <str> "
                         "(default='default_value')] [--help | -?]\n"
                         "Too few parameters (2 required)\n")
        self.assertEqual(out.getvalue(), '')
        sys_exit.assert_called_with(clingon.SYSTEM_EXIT_ERROR_CODE)

    def test_version_usage(self, sys_exit):
        with captured_output() as (out, err):
            clized_variables('p1')
        self.assertEqual(err.getvalue(),
                         "usage: test_clingon.py p1 p2 [options] [--help | -?]\n"
                         "Too few parameters (2 required)\n")
        self.assertEqual(out.getvalue(), '')
        sys_exit.assert_called_with(clingon.SYSTEM_EXIT_ERROR_CODE)

    def test_variables_help_output(self, sys_exit):
        with captured_output() as (out, err):
            clized_variables('-?')
        output = out.getvalue()
        doc = clized_default_shorts.docstring.strip().format(VERSION='1.2.3',
                                                           message="you can dynamically customize help message !")
        self.assertIn(doc, output)
        self.assertIn("Options:\n"
                      "--long-name-option | -l <str> (default='default_value')\n"
                      "--version          | -V print version (1.2.3)\n"
                      "--help             | -? print this help", output)
        self.assertEqual(err.getvalue(), '')
        sys_exit.assert_called_with(0)

    def test_version_one_short_usage(self, sys_exit):
        with captured_output() as (out, err):
            clized_variables_one_short('p1')
        self.assertEqual(err.getvalue(),
                         "usage: test_clingon.py p1 p2 [--option | -o <str> "
                         "(default='default_value')] [--version | -V] [--help | -?]\n"
                         "Too few parameters (2 required)\n")
        self.assertEqual(out.getvalue(), '')
        sys_exit.assert_called_with(clingon.SYSTEM_EXIT_ERROR_CODE)

    def test_version_parameter(self, sys_exit):
        with captured_output() as (out, err):
            clized_variables('-V')
        self.assertEqual(out.getvalue(), '')
        error = err.getvalue()
        self.assertIn('version 1.2.3 from', error)
        sys_exit.assert_called_with(0)


class TestMakeScript(unittest.TestCase):

    def call_script(self, target_path='', target_name='', user=False, make_link=False,
                force=False, remove=False, no_check_shebang=False, auto_install_clingon=False,
                args=()):
        return clingon.make_script(target_path, target_name, user, make_link,
                force, remove, no_check_shebang, auto_install_clingon, *args)

    def test_no_parameter(self):
        with captured_output() as (out, err):
            ret = self.call_script()
        self.assertEqual(ret, 1)
        self.assertEqual(out.getvalue(), "")
        self.assertEqual(err.getvalue(), "You must specify either a target script "
                                         "or --auto-install-clingon (-? for help)\n")

    def test_target_and_auto(self):
        with captured_output() as (out, err):
            ret = self.call_script(args=('clingon.py',), auto_install_clingon=True)
        self.assertEqual(ret, 1)
        self.assertEqual(out.getvalue(), "")
        self.assertEqual(err.getvalue(), "You cannot specify a target script and "
                                         "--auto-install-clingon at the same time (-? for help)\n")

    def test_multiple_target_file(self):
        with captured_output() as (out, err):
            ret = self.call_script(args=('clingon.py', 'toto.py'))
        self.assertEqual(ret, 1)
        self.assertEqual(out.getvalue(), "")
        self.assertEqual(err.getvalue(), "You must specify a single target, found: "
                                         "('clingon.py', 'toto.py') (-? for help)\n")

    def test_global_and_path(self):
        with captured_output() as (out, err):
            ret = self.call_script(args=('clingon.py',), user=True, target_path='/usr/local/bin')
        self.assertEqual(ret, 1)
        self.assertEqual(out.getvalue(), '')
        self.assertEqual(err.getvalue(), 'You cannot specify --path and --user at the same time (-? for help)\n')

    @mock.patch('os.path.exists')
    @mock.patch('os.unlink')
    def test_remove(self, os_unlink, os_path_exists):
        with captured_output() as (out, err):
            ret = self.call_script(args=('clingon.py',), remove=True, target_path='/usr/local/bin')
        self.assertIsNone(ret)
        self.assertEqual(out.getvalue(), "Script '/usr/local/bin/clingon' removed\n")
        self.assertEqual(err.getvalue(), '')
        os_unlink.assert_called_with('/usr/local/bin/clingon')
        os_path_exists.assert_called_with('/usr/local/bin/clingon')

    @mock.patch('os.path.exists', return_value=False)
    def test_remove_no_target(self, os_path_exists):
        with captured_output() as (out, err):
            ret = self.call_script(args=('clingon.py',), remove=True, target_path='/usr/local/bin')
        self.assertIsNone(ret)
        self.assertEqual(out.getvalue(), "Script '/usr/local/bin/clingon' not found, nothing to do\n")
        self.assertEqual(err.getvalue(), '')
        os_path_exists.assert_called_with('/usr/local/bin/clingon')

    @mock.patch('os.path.exists', return_value=False)
    def test_no_source(self, os_path_exists):
        with captured_output() as (out, err):
            ret = self.call_script(args=('toto.py',))
        self.assertEqual(ret, 1)
        self.assertEqual(out.getvalue(), "")
        self.assertIn('Could not find source', err.getvalue())
        self.assertIn("toto.py', aborting\n", err.getvalue())
        path = err.getvalue().split()[4][1:-2]
        os_path_exists.assert_called_with(path)

    @mock.patch('os.path.islink')
    @mock.patch('os.path.samefile')
    @mock.patch('os.path.exists')
    def test_target_exists_no_force(self, os_path_exists, os_path_samefile, os_path_islink):
        with captured_output() as (out, err):
            ret = self.call_script(args=('clingon.py',), target_path='/usr/local/bin')
        self.assertEqual(ret, 1)
        self.assertEqual(out.getvalue(), "")
        self.assertEqual(err.getvalue(), "Target '/usr/local/bin/clingon' already exists, aborting\n")

    @mock.patch('os.path.islink')
    @mock.patch('os.path.samefile')
    @mock.patch('os.path.exists')
    def test_target_created_aborting(self, os_path_exists, os_path_samefile, os_path_islink):
        with captured_output() as (out, err):
            ret = self.call_script(args=('clingon.py',), make_link=True, target_path='/usr/local/bin')
        self.assertIsNone(ret)
        self.assertEqual(out.getvalue(), "Target '/usr/local/bin/clingon' already created, nothing to do\n")
        self.assertEqual(err.getvalue(), "")

    @mock.patch('os.environ.get')
    @mock.patch('os.chmod')
    @mock.patch('os.stat', return_value=type('st', (object,), {'st_mode': 0}))
    @mock.patch('shutil.copyfile')
    @mock.patch('os.path.isdir')
    @mock.patch('os.path.islink')
    @mock.patch('os.path.samefile')
    @mock.patch('os.path.exists')
    @mock.patch('os.unlink')
    def test_copy_target(self, os_unlink, os_path_exists,
                                    os_path_samefile, os_path_islink, os_path_isdir,
                                    shutil_copyfile, os_stat, os_chmod, os_environ):
        with captured_output() as (out, err):
            ret = self.call_script(args=('clingon.py',), force=True, no_check_shebang=True, target_path='/usr/local/bin')
        self.assertIsNone(ret)
        self.assertIn("has been copied to /usr/local/bin/clingon", out.getvalue())
        self.assertEqual(err.getvalue(), "")
        os_path_isdir.assert_called_with('/usr/local/bin')
        os_unlink.assert_called_with('/usr/local/bin/clingon')
        os_chmod.assert_called_with('/usr/local/bin/clingon', 72)

    @mock.patch('os.environ.get', return_value='/usr/local/bin')
    @mock.patch('os.chmod')
    @mock.patch('os.stat', return_value=type('st', (object,), {'st_mode': 0}))
    @mock.patch('os.symlink')
    @mock.patch('os.path.isdir')
    @mock.patch('os.path.islink')
    @mock.patch('os.path.samefile', return_value=False)
    @mock.patch('os.path.exists')
    @mock.patch('os.unlink')
    def test_symlink_target(self, os_unlink, os_path_exists,
                                    os_path_samefile, os_path_islink, os_path_isdir,
                                    os_symlink, os_stat, os_chmod, os_environ):
        with captured_output() as (out, err):
            ret = self.call_script(args=('clingon.py',), force=True, make_link=True,
                                      no_check_shebang=True, target_path='/usr/local/bin')
        self.assertIsNone(ret)
        self.assertIn("has been symlinked to /usr/local/bin/clingon", out.getvalue())
        self.assertEqual(err.getvalue(), "")
        os_path_isdir.assert_called_with('/usr/local/bin')
        os_unlink.assert_called_with('/usr/local/bin/clingon')
        os_chmod.assert_called_with('/usr/local/bin/clingon', 72)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read().replace('.. :changelog:', '')

requirements = ['future', 'wheel']
test_requirements = ['mock', 'coverage', 'nose', 'nose-cov', 'codecov']

# Add Python 2.6 specific dependencies
if sys.version_info[:2] < (2, 7):
    test_requirements.append('unittest2')
    requirements.append('ordereddict')


setup(
    name='clingon',
    version='0.1.1',
    description="Command Line INterpreter Generator for pythON",
    long_description=readme + '\n\n' + history,
    author="François Vincent",
    author_email='francois.vincent01@gmail.com',
    url='https://github.com/francois-vincent/clingon',
    packages=[
        'clingon',
    ],
    package_dir={'clingon':
                 'clingon'},
    include_package_data=True,
    install_requires=requirements,
    license="BSD",
    zip_safe=False,
    keywords='clingon',
    classifiers=[
        # 'Development Status ::  3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Topic :: Desktop Environment',
        'Topic :: System :: Shells',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Programming Language :: Python :: Implementation :: CPython'
    ],
    test_suite='tests',
    tests_require=test_requirements
)
